/***************************************************************************
                          lcd.c  -  Handle LCD on Compaq Armada E500
                             -------------------
    begin                : Wed Jun 22 2005
    copyright            : (C) 2005 by Andrei Gavrila
    email                : andreig@revolutionstudio.ro
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "lcd.h"

void out_idx(int reg, int val)
{
  outb(reg, INDEX_PORT);
  outb(val, DATA_PORT);
}

unsigned int in_idx(int reg)
{
  outb(reg, INDEX_PORT);

  return inb(DATA_PORT);
}

int lcdGetBrightness(void)
{
  int    id, n, brightness;

  if (iopl(3) == -1) {
    perror("iopl");
    return -1;
  }

  outb(CONF_START,INDEX_PORT);
  id = in_idx(REG_CHIP_ID);

  if ((id != 0xa) && (id != 0xb)) {
    fprintf(stderr,"unknown chip id: %02x\n",id);
    return -1;
  }

  out_idx(REG_LOGDN, 9);
  n = (in_idx(REG_LDN_BASEH) << 8) + in_idx(REG_LDN_BASEL);

  outb(MBX_PWM1, n);

  brightness = inb(n + 1) >> 1;

  return brightness >> 3;
}

int lcdSetBrightness(int brightness)
{
  int    id, n;

  brightness = brightness > 7 ? 7 : brightness;
  brightness = brightness < 0 ? 0 : brightness;

  if (iopl(3) == -1) {
    perror("iopl");
    return -1;
  }

  outb(CONF_START,INDEX_PORT);
  id = in_idx(REG_CHIP_ID);

  if ((id != 0xa) && (id != 0xb)) {
    fprintf(stderr,"unknown chip id: %02x\n",id);
    return -1;
  }

  out_idx(REG_LOGDN, 9);
  n = (in_idx(REG_LDN_BASEH) << 8) + in_idx(REG_LDN_BASEL);

  outb(brightness*8*2, n + 1);
  outb(CONF_END,INDEX_PORT);

  return brightness;
}
